from google.cloud import datastore
import os


class DatastoreClient:
    class __DatastoreClient:
        def __init__(self):
            self.client = datastore.Client(os.environ.get("PROJECT"))

    instance = None

    def __init__(self):
        if not DatastoreClient.instance:
            DatastoreClient.instance = DatastoreClient.__DatastoreClient()

    def get_client(self):
        return self.client

    def __getattr__(self, name):
        return getattr(self.instance, name)



from google.cloud import pubsub
import os


class Publisher:
    class __Publisher:
        def __init__(self):
            self.publish_client = pubsub.PublisherClient()

    instance = None

    def __init__(self):
        if not Publisher.instance:
            Publisher.instance = Publisher.__Publisher()

    def publish_messages(self, topic, messages=None, message=None):
        topic_name = 'projects/{project_id}/topics/{topic}'.format(
            project_id=os.environ.get("PROJECT"),
            topic=topic,
        )
        if messages:
            for message in messages:
                self.publish_client.publish(topic_name, message)
        elif message:
            self.publish_client.publish(topic_name, message)

    def __getattr__(self, name):
        return getattr(self.instance, name)


import pytest
from publisher import Publisher


def test_project_must_exists():
    client = Publisher('merlin-qa')

    assert isinstance(client, Publisher)


def test_instance_without_project_must_fail():
    with pytest.raises(Exception):
        Publisher()



from .publisher import Publisher
from .util import firebase_auth, validate_email, build_statistic_event, emails_substitutions, algolia_request
from .constants import Constants
from .datastore import DatastoreClient

name = "merlin_utils"


ds_client = DatastoreClient()
constants = Constants()
publisher = Publisher()



import copy
import logging
import os
import re
from datetime import datetime
from functools import wraps

from .constants import Constants
from flask import request
from flask_api import status
from google.auth.transport import requests
from google.cloud.datastore import Entity
from google.oauth2 import id_token
from uuid import uuid4

HTTP_REQUEST = requests.Request()
PROJECT_ID = os.environ.get('PROJECT')
UNAUTH = {"Message": "User has not permissions"}, status.HTTP_401_UNAUTHORIZED
constants = Constants()


def firebase_auth(f):
    @wraps(f)
    def wrapped(*args, **kwargs):
        id_token_firebase = request.headers.get('Authorization')
        if not id_token_firebase:
            return UNAUTH
        try:
            claims = id_token.verify_firebase_token(id_token_firebase.split(' ').pop(), HTTP_REQUEST)
        except Exception as ex:
            logging.warning(ex)
            return UNAUTH
        logging.info(claims)
        if not (claims.get('aud') == PROJECT_ID or claims.get('iss') == os.environ.get('ISSUER') % PROJECT_ID):
            return UNAUTH
        kwargs["auth_user"] = {"id": claims.get('sub'), "role": claims.get('role')}
        return f(*args, **kwargs)

    return wrapped


def pre_loader(in_data):
    if in_data is not None:
        if isinstance(in_data, Entity):
            in_data["key"] = in_data.key
        for key, value in copy.deepcopy(in_data).items():
            if isinstance(value, datetime):
                in_data[key] = value.isoformat()
            elif value is None:
                in_data.pop(key)


def validate_email(email):
    if email is not None:
        return email.split("@")[0] if re.fullmatch(
            "^[_a-z0-9-+]+(\.[_a-z0-9-]+)*@[a-z0-9-+]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$", email) is not None else None


def build_statistic_event(event, id_user, role, id_receptor=None, id_job=None, extra_params=None):
    statistic_event = {
        "idEvent": str(uuid4()),
        "idUser": id_user,
        "userRole": role,
        **event
    }
    id_sender_name = "idUserCandidate" if role == "Candidate" else "idUserEmployer"
    statistic_event[id_sender_name] = id_user
    if id_receptor:
        id_receptor_name = "idUserEmployer" if role == "Candidate" else "idUserCandidate"
        statistic_event[id_receptor_name] = id_receptor
    if extra_params:
        statistic_event["properties"].update(extra_params)
    if id_job:
        statistic_event["idJob"] = id_job

    return {"event": statistic_event, "appsflyer": statistic_event.get("appsflyer")}


def emails_substitutions(substitutions, id_user):
    emails_substitution = {
        "{{ asm_group_unsubscribe_raw_url }}": "www.merlinjobs.com/unsubscribe?id={}".format(id_user),
        "{{ unsubscribe_sendgrid_default }}": "<%asm_group_unsubscribe_raw_url%>",
        "https://www.merlinjobs.com/": constants.get_constant("FINAL_URL"),
        **substitutions
    }
    return emails_substitution


def algolia_request(obj_type, obj):
    search_request = {
        "algolia": True,
        "body": {
            "newImplementation": True,
            "keyClient": constants.get_constant("KEY_SECURE_CLIENT"),
            "{}List".format(obj_type): [obj]
        },
        "url": "/{}Update".format(obj_type)
    }
    return search_request



import copy

from . import ds_client
from google.cloud import datastore
from google.cloud.datastore import Entity


class BaseModel():
    def __init__(self, **kwargs):
        self.kind = kwargs.get("kind", None)
        self.id = kwargs.get("id", None)
        self.key = kwargs.get("key", None)
        self.parent = kwargs.get("parent", None)
        self.exclude_indexes = kwargs.get("exclude_indexes", None)

    def save(self):
        if self.key is not None:
            entity = datastore.Entity(self.key)
            entity.update(self.__dict__)
            ds_client.get_client().put(entity)

    @staticmethod
    def get(id):
        """
        :param id: This method only works for websafe ids
        :return: Entity
        """
        return ds_client.get_client().get(datastore.Key.from_legacy_urlsafe(id))

    def to_entity(self):
        if self.key is None:
            self.key = ds_client.get_client().key(self.kind, self.id) if self.parent is None else ds_client.get_client() \
                .key(self.parent[0], self.parent[1], self.kind, self.id)
        entity = Entity(self.key, exclude_from_indexes=self.exclude_indexes)
        props = copy.deepcopy(self.__dict__)
        props.pop("kind", None)
        props.pop("id", None)
        props.pop("key", None)
        props.pop("parent", None)
        props.pop("exclude_indexes")
        entity.update(props)
        return entity



from google.cloud import datastore
import os


class Constants:
    class __Constants:
        def __init__(self):
            datastore_client = datastore.Client(os.environ.get("PROJECT"))
            query = datastore_client.query(kind="Params")
            query.add_filter("status", "=", "ACTIVE")
            constants = {}
            for entity in list(query.fetch()):
                value = entity.get("langValues", entity.get("value"))
                constants[entity["name"]] = value
            self.constants = constants

    instance = None

    def __init__(self):
        if not Constants.instance:
            Constants.instance = Constants.__Constants()

    def get_constant(self, key, language="en"):
        constant = self.constants.get(key)
        if constant is not None:
            return constant.get(language) if isinstance(constant, dict) else constant

    def __getattr__(self, name):
        return getattr(self.instance, name)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

REQUIRED_PACKAGES = [
    "Flask==1.0.2",
    "google-api-python-client==1.7.3",
    "google-auth==1.5.0",
    "requests==2.18.4",
    "oauth2client==4.1.2",
    "Flask-api==1.0.0",
    "google-cloud-datastore==1.7.0",
    "google-cloud-pubsub==0.37.0",
]

setuptools.setup(
    name="merlin_utils",
    version="0.0.23",
    author="Merlin Jobs inc.",
    author_email="oficina@merlinjobs.com",
    description="Merlin utils",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://http//github.com/merlinapp/merlin_python_utils",
    install_requires=REQUIRED_PACKAGES,
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)
